import os
from stresser.modules import *

my_config = parse_input()

SOLUTION_PATH = my_config['SOLUTION_PATH']
DUMMY_PATH = my_config['DUMMY_PATH']
GENERATOR_PATH = my_config['GENERATOR_PATH']
NTESTS = my_config['NTESTS']
SAVE_TESTS = my_config['SAVE_TESTS']
CHECKER_TYPE = my_config['CHECKER_TYPE']

if not os.path.exists('stresser_data'):
    os.mkdir('stresser_data')
if not os.path.exists(f"stresser_data{os.sep}files"):
    os.mkdir(f"stresser_data{os.sep}files")
if not os.path.exists(f"stresser_data{os.sep}tests"):
    os.mkdir(f"stresser_data{os.sep}tests")
clean(tests=True)
clean(files=True)


launch_solution = get_launcher(SOLUTION_PATH, "solution")
launch_dummy = get_launcher(DUMMY_PATH, "dummy")
launch_generator = get_launcher(GENERATOR_PATH, "generator")

cur_checker = None
if CHECKER_TYPE == 'base':
    cur_checker = base_check
elif CHECKER_TYPE == 'base_with_format':
    cur_checker = base_with_format_check


for test_num in range(1, NTESTS + 1):
    if not SAVE_TESTS:
        clean(tests=True)
    test_path = f"stresser_data{os.sep}tests{os.sep}{test_num}_in.txt"
    dummy_out_path = f"stresser_data{os.sep}tests{os.sep}{test_num}_out_dummy.txt"
    solution_out_path = f"stresser_data{os.sep}tests{os.sep}{test_num}_out_solution.txt"

    launch_generator(out_path=test_path, flags=str(test_num))
    launch_dummy(test_path, dummy_out_path)
    launch_solution(test_path, solution_out_path)

    f_dummy_out = open(dummy_out_path, "r")
    dummy_out = f_dummy_out.read()
    f_dummy_out.close()

    f_solution_out = open(solution_out_path, "r")
    solution_out = f_solution_out.read()
    f_solution_out.close()

    

    if cur_checker(dummy_out, solution_out):
        print(f"test {test_num} OK")
    else:
        target_test = open(test_path)
        print(
            f"""
test {test_num} WA
------------------------------
INPUT:
{target_test.read()}
--------------------
RIGHT:
{dummy_out}
--------------------
WRONG:
{solution_out}
------------------------------
    """
        )
        target_test.close()
        break

if not SAVE_TESTS:
    clean(files=True, tests=True)


import os
from argparse import ArgumentParser
from configparser import ConfigParser

CHECKER_TYPES = ['base', 'base_with_format']

def validate(in_data: dict):
    for now_path in ['SOLUTION_PATH', 'DUMMY_PATH', 'GENERATOR_PATH']:
        assert (in_data[now_path] != None), f"there is not {now_path}"
        # if not os.path.isabs(in_data[now_path]):
        #     in_data[now_path] = os.path.abspath(in_data[now_path])
        assert os.path.isfile(in_data[now_path]), in_data[now_path] + ': not such file'
    
    assert (type(in_data["NTESTS"]) == int) and (in_data["NTESTS"] > 0), "NTESTS must be positive integer"
    assert (in_data['CHECKER_TYPE'] in CHECKER_TYPES), "wrong CHECKER_TYPE"

def set_default(in_data: dict):
    def_dict = {
        'NTESTS': 10000,
        'SAVE_TESTS': False,
        'CHECKER_TYPE': 'base_with_format',
        'SOLUTION_PATH': None,
        'DUMMY_PATH': None,
        'GENERATOR_PATH': None
    }
    for key, val in in_data.items():
        if val != None:
            def_dict[key] = val
    
    return def_dict


def parse_input() -> dict:
    parser = ArgumentParser()

    parser.add_argument("-c", "--CONFIG_PATH", help="путь к конфигурационному файлу.\
    Если прописан, то все аргументы будут браться из него")
    parser.add_argument("-S", "--SOLUTION_PATH", help="путь к решению с ошибками")
    parser.add_argument("-D", "--DUMMY_PATH", help="путь к правильному решению")
    parser.add_argument("-G", "--GENERATOR_PATH", help="путь к программе, которая генерирует данные в stdout")
    parser.add_argument("-N", "--NTESTS", type=int, help="необходимое количество тестов. Базовое значение: 10000")
    parser.add_argument("-s", "--SAVE_TESTS", help="сохранять ли тесты. варианты: true/false. Базовое значение: false")
    parser.add_argument("-t", "--CHECKER_TYPE", help=f"тип чекера.\
    варианты: {CHECKER_TYPES} Базовое значение: base_with_format")
    args = parser.parse_args()
    res_dict = {}
    if args.CONFIG_PATH == None:
        res_dict = vars(args)
        res_dict.pop('CONFIG_PATH')
    else:
        config = ConfigParser()
        config.optionxform = str
        config.read(os.path.abspath(args.CONFIG_PATH))
        res_dict = (dict(config['settings']))
        res_dict['NTESTS'] = config.getint('settings', 'NTESTS')
        res_dict['SAVE_TESTS'] = config.getboolean('settings', 'SAVE_TESTS')
    res_dict = set_default(res_dict)
    validate(res_dict)
    return res_dict



def base_check(dummy_ans: str, solution_ans: str) -> bool:
    return dummy_ans == solution_ans


def make_format(s: str) -> str: 
    s = s.strip()
    s = s.replace(' \n', '\n')
    return s

def base_with_format_check(dummy_ans: str, solution_ans: str) -> bool:
    s1 = make_format(dummy_ans)
    s2 = make_format(solution_ans)
    return s1 == s2
